"""
    A variety of methods used to aid parsing, testing
"""
__author__="sdrobert"
from re import compile as rec, escape
from string import punctuation

def sanitizeAndValidateLexicon(lexiconFileName):
    """
        Returns a pair. The latter element is the "sanitized" content of a
        lexicon (without comments), and the former element is a boolean
        specifying if the lexicon is a valid lexicon (only contains terminals)
        
        :param lexiconFileName: a valid path to the file storing terminal rules
    """
    
    return _sanitizeAndValidateWithPattern(lexiconFileName,
        rec('^[^\'"]+\s+->\s+(?P<quote>[\'"])[^\'"]+(?P=quote)'
            '(\s*\\|\s*(?P=quote)[^\'"]+(?P=quote))*$'))

def sanitizeAndValidateGrammar(grammarFileName):
    """
        Returns a pair. The latter element is the "sanitized" content of a
        grammar (without comments), and the former element is a boolean
        specifying if the grammar is a valid one (only contains nonterminals)
    """
    
    return _sanitizeAndValidateWithPattern(grammarFileName,
        rec('^[^\'"]+\s+->\s+[^\'"]+((\s*\|)|\s+[^\'"]+)*$'))

def sanitizeAndValidateSentences(sentenceFileName):
    """
        Returns a pair. The latter element is the "sanitized" content of the
        test sentences (without comments) and the former element is a boolean
        specifying if the sentences are in the proper format (no punctuation,
        possible asterisk in front specifying no parse)
    """
    return _sanitizeAndValidateWithPattern(sentenceFileName,
        rec('^\*?[^'+escape(punctuation)+']+$'))

def _sanitizeAndValidateWithPattern(fileName,pattern):
    # "private" method
    assert type(fileName) is str
    sanitized = []
    isValid = True
    # this pattern isn't a perfect match for a lexicon parse, but should
    # cover the average case. Let me know if there's a case that works in
    # parse_cfg but not here that you want covered.
    with open(fileName,'r') as f:
        for line in f:
            # remove comments and whitespace
            sl = line.split('%')[0].strip()
            if not sl:
                continue # blank line
            if not pattern.match(sl):
                print('Failed to parse line "{}" in {}. Are you sure'
                    ' it\'s in the right file?'.format(sl,fileName))
                isValid = False
            
            sanitized.append(sl)
    
    return (isValid,'\n'.join(sanitized))
